/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
*/

/*
  Copyright (C) 2007 Eike Hein <hein@kde.org>
*/


#include "terminal_focus_watcher.h"
#include "terminal_focus_watcher.moc"


TerminalFocusWatcher::TerminalFocusWatcher(TQObject* parent, const char* name)
    : TQObject(parent, name)
{
}

TerminalFocusWatcher::~TerminalFocusWatcher()
{
}

bool TerminalFocusWatcher::eventFilter (TQObject* /* watched */, TQEvent* e)
{
    if (e->type() == TQEvent::FocusIn)
        emit focusChanged();

    return false;
}
